@@ -79,9 +79,21 @@ static int safe_write(int fd, const char *buf, int len);
 static char *parse_cmd(const char *cmd, const char *event);
 static int check_escapes(const char *str);
 
+/* helper */
+static int
+path_is_dir(const char *path)
+{
+	struct stat s;
+
+	if (stat(path, &s)) {
+		acpid_log("ERR: stat(\"%s\"): %s\n", path, strerror(errno));
+		return 0;
+	}
+	return S_ISDIR(s.st_mode);
+}
 
 /*
- * read in all the configuration files 
+ * read in all the configuration files
  */
 int
 acpid_read_conf(const char *confdir)
@@ -90,12 +102,12 @@ acpid_read_conf(const char *confdir)
 	struct dirent *dirent;
 	char *file = NULL;
 	int nrules = 0;
-	
+
 	lock_rules();
 
 	dir = opendir(confdir);
 	if (!dir) {
-		acpid_log("ERR: opendir(%s): %s\n", 
+		acpid_log("ERR: opendir(%s): %s\n",
 			confdir, strerror(errno));
 		unlock_rules();
 		return -1;
@@ -105,6 +117,7 @@ acpid_read_conf(const char *confdir)
 	while ((dirent = readdir(dir))) {
 		int len;
 		struct rule *r;
+
 		if (dirent->d_name[0] == '.')
 			continue; /* skip dotfiles */
 
@@ -116,6 +129,12 @@ acpid_read_conf(const char *confdir)
 			return -1;
 		}
 		snprintf(file, len, "%s/%s", confdir, dirent->d_name);
+
+		if (path_is_dir(file)) {
+			free(file);
+			continue; /* skip subdirs */
+		}
+
 		r = parse_file(file);
 		if (r) {
 			enlist_rule(&cmd_list, r);
@@ -129,7 +148,7 @@ acpid_read_conf(const char *confdir)
 	acpid_log("%d rule%s loaded\n", nrules, (nrules == 1)?"":"s");
 
 	return 0;
-}	
+}
 
 /*
  * cleanup all rules
@@ -169,8 +188,8 @@ acpid_cleanup_rules(void)
 
 	return 0;
 }
-	
-static struct rule * 
+
+static struct rule *
 parse_file(const char *file)
 {
 	FILE *fp;
@@ -230,7 +249,7 @@ parse_file(const char *file)
 		/* quick parse */
 		n = sscanf(p, "%63[^=\n]=%255[^\n]", key, val);
 		if (n != 2) {
-			acpid_log("can't parse %s at line %d\n", 
+			acpid_log("can't parse %s at line %d\n",
 				file, line);
 			continue;
 		}
@@ -242,7 +261,7 @@ parse_file(const char *file)
 			int rv;
 			r->event = malloc(sizeof(regex_t));
 			if (!r) {
-				acpid_log("ERR: malloc(): %s\n", 
+				acpid_log("ERR: malloc(): %s\n",
 					strerror(errno));
 				free_rule(r);
 				fclose(fp);
@@ -252,7 +271,7 @@ parse_file(const char *file)
 			if (rv) {
 				char buf[128];
 				regerror(rv, r->event, buf, sizeof(buf));
-				acpid_log("ERR: regcomp(): %s\n", buf); 
+				acpid_log("ERR: regcomp(): %s\n", buf);
 				free_rule(r);
 				fclose(fp);
 				return NULL;
@@ -266,7 +285,7 @@ parse_file(const char *file)
 			}
 			r->action.cmd = strdup(val);
 			if (!r->action.cmd) {
-				acpid_log("ERR: strdup(): %s\n", 
+				acpid_log("ERR: strdup(): %s\n",
 					strerror(errno));
 				free_rule(r);
 				fclose(fp);
@@ -333,7 +352,7 @@ parse_client(int client)
 	if (rv) {
 		char buf[128];
 		regerror(rv, r->event, buf, sizeof(buf));
-		acpid_log("ERR: regcomp(): %s\n", buf); 
+		acpid_log("ERR: regcomp(): %s\n", buf);
 		free_rule(r);
 		return NULL;
 	}
@@ -358,7 +377,7 @@ enlist_rule(struct rule_list *list, struct rule *r)
 	}
 }
 
-static void 
+static void
 delist_rule(struct rule_list *list, struct rule *r)
 {
 	if (r->next) {
@@ -397,7 +416,7 @@ new_rule(void)
 }
 
 /* I hope you delisted the rule before you free() it */
-static void 
+static void
 free_rule(struct rule *r)
 {
 	if (r->type == RULE_CMD) {
@@ -417,7 +436,7 @@ free_rule(struct rule *r)
 	free(r);
 }
 
-/* 
+/*
  * the main hook for propogating events
  */
 int
@@ -427,7 +446,7 @@ acpid_handle_event(const char *event)
 	int nrules = 0;
 	struct rule_list *ar[] = { &client_list, &cmd_list, NULL };
 	struct rule_list **lp;
-	
+
 	/* make an event be atomic wrt known signals */
 	lock_rules();
 
@@ -466,7 +485,7 @@ acpid_handle_event(const char *event)
 	unlock_rules();
 
 	if (acpid_debug) {
-		acpid_log("DBG: %d total rule%s matched\n", 
+		acpid_log("DBG: %d total rule%s matched\n",
 			nrules, (nrules == 1)?"":"s");
 	}
 
@@ -477,11 +496,8 @@ acpid_handle_event(const char *event)
 static sigset_t *
 signals_handled(void)
 {
-	static int been_here;
 	static sigset_t sigset;
 
-	been_here = 1;
-
 	sigemptyset(&sigset);
 	sigaddset(&sigset, SIGHUP);
 	sigaddset(&sigset, SIGTERM);
@@ -530,11 +546,14 @@ do_cmd_rule(struct rule *rule, const char *event)
 		action = parse_cmd(rule->action.cmd, event);
 		acpid_log("executing action \"%s\"\n", action);
 
+		/* reset signals */
 		signal(SIGHUP, SIG_DFL);
 		signal(SIGTERM, SIG_DFL);
 		signal(SIGINT, SIG_DFL);
 		signal(SIGQUIT, SIG_DFL);
 		signal(SIGPIPE, SIG_DFL);
+		sigprocmask(SIG_UNBLOCK, signals_handled(), NULL);
+
 		acpid_log("BEGIN HANDLER MESSAGES\n");
 		execl("/bin/sh", "/bin/sh", "-c", action, NULL);
 		/* should not get here */
@@ -546,7 +565,7 @@ do_cmd_rule(struct rule *rule, const char *event)
 	waitpid(pid, &status, 0);
 	acpid_log("END HANDLER MESSAGES\n");
 	if (WIFEXITED(status)) {
-		acpid_log("action exited with status %d\n", 
+		acpid_log("action exited with status %d\n",
 			WEXITSTATUS(status));
 	} else if (WIFSIGNALED(status)) {
 		acpid_log("action exited on signal %d\n", WTERMSIG(status));
@@ -556,13 +575,13 @@ do_cmd_rule(struct rule *rule, const char *event)
 
 	return 0;
 }
-		
-static int 
+
+static int
 do_client_rule(struct rule *rule, const char *event)
 {
 	int r;
 	int client = rule->action.fd;
-	
+
 	acpid_log("notifying client %s\n", rule->origin);
 
 	r = safe_write(client, event, strlen(event));
@@ -642,7 +661,7 @@ parse_cmd(const char *cmd, const char *event)
 	if (acpid_debug >= 2) {
 		acpid_log("DBG: expanded \"%s\" -> \"%s\"\n", cmd, buf);
 	}
-	
+
 	return buf;
 }
 
