@@ -1,5 +1,5 @@
 /*
- * $Id: ud_socket.c,v 1.3 2001/08/17 18:09:40 sunthockin Exp $
+ * $Id: ud_socket.c,v 1.4 2003/11/17 21:24:58 sunthockin Exp $
  * A few  routines for handling UNIX domain sockets
  */
 
@@ -17,7 +17,7 @@
 #include "acpid.h"
 #include "ud_socket.h"
 
-int 
+int
 ud_create_socket(const char *name)
 {
 	int fd;
@@ -77,3 +77,29 @@ ud_accept(int listenfd, struct ucred *cred)
 		return newsock;
 	}
 }
+
+int
+ud_connect(const char *name)
+{
+	int fd;
+	int r;
+	struct sockaddr_un addr;
+
+	fd = socket(AF_UNIX, SOCK_STREAM, 0);
+	if (fd < 0) {
+		return fd;
+	}
+
+	memset(&addr, 0, sizeof(addr));
+	addr.sun_family = AF_UNIX;
+	sprintf(addr.sun_path, "%s", name);
+
+	r = connect(fd, (struct sockaddr *)&addr, sizeof(addr));
+	if (r < 0) {
+		close(fd);
+		return r;
+	}
+
+	return fd;
+}
+
